import FoodRestrictionRepository from '../Repositories/FoodRestrictionRepository.js'
import FilhosController from './FilhosController.js'
import { log } from './LogController.js'

const getFoodRestriction = async function ({CDCONSUMIDOR, consDetails}) {
    let result = await FoodRestrictionRepository.getFoodRestriction(consDetails, CDCONSUMIDOR)

    const idpaiList = result.map(m => m.IDPAI)
    const foodRestriction = {}

    idpaiList.forEach(id => {
        const currPaiMov = result.filter(m => m.IDPAI === id)
        const foodRestrictionPai = []
        if (currPaiMov.length) {
            for (let i = 0; i < result.length; i++) {
                if (result[i].IDPAI === id)
                foodRestrictionPai.push(result[i])
            }
        }
    
        const nomePai = result.filter(m => m.ID == id)[0]
        if (nomePai && nomePai.DSRESTRICAO)
            foodRestriction[nomePai.DSRESTRICAO] = foodRestrictionPai
    })      

    return foodRestriction
}

const updateFoodRestriction = async function ({CDCONSUMIDOR, QTCONSDIARAL, IDPAI, CDPRODUTO, cdWeekDays, consDetails}) {
    let result = await FoodRestrictionRepository.updateFoodRestriction(consDetails, CDCONSUMIDOR, QTCONSDIARAL, IDPAI, CDPRODUTO, cdWeekDays)

    return result
}

const getConsumerFamilies = async function ({consDetails}) {
    let result = await FoodRestrictionRepository.getConsumerFamilies(consDetails)

    return result
}

const getFamilyProducts = async function ({CDCONSUMIDOR, CDFAMILISALD, consDetails}) {
    let result = await FoodRestrictionRepository.getFamilyProducts(consDetails, CDCONSUMIDOR, CDFAMILISALD)

    return result
}

const addFoodRestriction = async function ({CDCONSUMIDOR, productCodes, CDFAMILISALD, QTCONSDIARAL, cdWeekDays, consDetails}) {
    // get all branches associated with the current client
    let branchList = await FoodRestrictionRepository.getClientBranchList(consDetails.CDCLIENTE)
    let responseData = {
        rowsAffected: 0
    }

    if (branchList?.length > 0) {
        // do a loop for each branch found and register one restriction for each one
        for (let j = 0; j < branchList.length; j++) {
            const element = branchList[j]

            for (let i = 0; i < productCodes.length; i++) {
                /* Checks if the family associated with the product already has a value restriction. */
                let valueRestriction = await FoodRestrictionRepository.getValueRestriction(consDetails, CDCONSUMIDOR, CDFAMILISALD, element.CDFILIAL)

                if (valueRestriction.length == 0) {
                    await FoodRestrictionRepository.addValueRestriction(consDetails, CDCONSUMIDOR, CDFAMILISALD, null, element.CDFILIAL)
                }

                let result = await FoodRestrictionRepository.addFoodRestriction(consDetails, CDCONSUMIDOR, productCodes[i],
                    CDFAMILISALD, QTCONSDIARAL, cdWeekDays, element.CDFILIAL, consDetails.CDCLIENTE)

                if (result?.rowsAffected > 0) responseData.rowsAffected += result.rowsAffected
            }   
        }
    }

    return responseData
}

const deleteFoodRestriction = async function ({CDCONSUMIDOR, CDFAMILISALD, CDPRODUTO, consDetails}) {
    let result = await FoodRestrictionRepository.deleteFoodRestriction(consDetails, CDCONSUMIDOR, CDFAMILISALD, CDPRODUTO)

    return result
}

const replicateProductRestrictions = async (CDFILIAL, consDetails) => {
    try {
        let getFilhosParams = {
            CDCLIENTE: consDetails.CDCLIENTE,
            NRCPFRESPCON: consDetails.NRCPFRESPCON,
            CDFILIAL
        }
        let consumerArray = await FilhosController.getFilhos(getFilhosParams)
        let branchList = await FoodRestrictionRepository.getClientBranchList(consDetails.CDCLIENTE)
        
        await replicateRestrictionsForEachConsumer(consumerArray, branchList, CDFILIAL, consDetails)
    } catch (error) {
        log({ type: 'error', message: `Ocorreu um erro no processo de replicar as restricoes alimentares:\n${error.message}` })
    }
}

const replicateRestrictionsForEachConsumer = async (consumerArray, branchList, CDFILIAL, consDetails) => {
    for (let i = 0; i < consumerArray.length; i++) {
        const currentConsumer = consumerArray[i]
        let productRestrictions = await FoodRestrictionRepository.getConsumerProductRestrictions(CDFILIAL, consDetails.CDCLIENTE, currentConsumer.CDCONSUMIDOR)

        for (let j = 0; j < productRestrictions.length; j++) {
            const currentRestriction = productRestrictions[j]

            for (let k = 0; k < branchList.length; k++) {
                const currentBranch = branchList[k]
                await replicateProductRestriction(currentBranch.CDFILIAL, consDetails.CDCLIENTE, currentConsumer.CDCONSUMIDOR,
                    currentRestriction.CDPRODUTO, currentRestriction.CDFAMILISALD, currentRestriction.QTCONSDIARAL, currentRestriction.IDBLOQDIARAL)
            }
        }
    }
}            

const replicateProductRestriction = async (CDFILIAL, CDCLIENTE, CDCONSUMIDOR, CDPRODUTO, CDFAMILISALD, QTCONSDIARAL, IDBLOQDIARAL) => {
    let checkRestriction = await FoodRestrictionRepository.checkProductRestriction(CDFILIAL, CDCLIENTE, CDCONSUMIDOR, CDPRODUTO, CDFAMILISALD)

    if (checkRestriction.length === 0) {
        /* Checks if the family associated with the product already has a restriction. */
        let valueRestriction = await FoodRestrictionRepository.getValueRestriction({ CDCLIENTE }, CDCONSUMIDOR, CDFAMILISALD, CDFILIAL)

        if (valueRestriction.length === 0) {
            await FoodRestrictionRepository.addValueRestriction({ CDCLIENTE }, CDCONSUMIDOR, CDFAMILISALD, null, CDFILIAL)
        }

        await FoodRestrictionRepository.addFoodRestriction(null, CDCONSUMIDOR, CDPRODUTO, CDFAMILISALD, QTCONSDIARAL, IDBLOQDIARAL, CDFILIAL, CDCLIENTE)
    }
}

export default {
    getFoodRestriction,
    updateFoodRestriction,
    getConsumerFamilies,
    getFamilyProducts,
    addFoodRestriction,
    deleteFoodRestriction,
    replicateProductRestrictions
}